@@ -5,7 +5,7 @@ use term::*;
 use term::Term::*;
 use term::Error::*;
 use booleans::*;
-use pair::{pair, second};
+use pair::pair;
 use combinators::{i, k, z};
 
 /// Produces a Church-encoded number zero.
@@ -35,8 +35,8 @@ pub fn zero() -> Term { abs(abs(Var(1))) }
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(is_zero(), 0.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(is_zero(), 1.into()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(is_zero(), 0.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(is_zero(), 1.into()), NOR, 0), fls());
 /// # }
 /// ```
 pub fn is_zero() -> Term {
@@ -72,7 +72,7 @@ pub fn one() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = succ().app(0.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 1.into());
 /// ```
@@ -94,7 +94,7 @@ pub fn succ() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(plus(), 3.into(), 2.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 5.into());
 /// # }
@@ -105,7 +105,7 @@ pub fn plus() -> Term {
     ))))
 }
 
-/// Applied to two Church-encoded numbers it produces their product.
+/// Applied to two Church-encoded numbers it yields their product.
 ///
 /// MULT := λmnf.m (n f) = λ λ λ 3 (2 1)
 ///
@@ -117,7 +117,7 @@ pub fn plus() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(mult(), 2.into(), 3.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 6.into());
 /// # }
@@ -130,7 +130,7 @@ pub fn mult() -> Term {
 
 /// Applied to two Church-encoded numbers it raises the first one to the power of the second one.
 ///
-/// POW := λbe.IS_ZERO e ONE (e b) = λ λ IS_ZERO 1 ONE (1 2)
+/// POW := λab.IS_ZERO b ONE (b a) = λ λ IS_ZERO 1 ONE (1 2)
 ///
 /// # Example
 /// ```
@@ -140,7 +140,7 @@ pub fn mult() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(pow(), 2.into(), 3.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 8.into());
 /// # }
@@ -161,7 +161,7 @@ pub fn pow() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = pred().app(3.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 2.into());
 /// ```
@@ -178,7 +178,7 @@ pub fn pred() -> Term {
 
 /// Applied to two Church-encoded numbers it subtracts the second one from the first one.
 ///
-/// SUB := λmn.n PRED m = λ λ 1 PRED 2
+/// SUB := λab.b PRED a = λ λ 1 PRED 2
 ///
 /// # Example
 /// ```
@@ -188,7 +188,7 @@ pub fn pred() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(sub(), 5.into(), 3.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 2.into());
 /// # }
@@ -213,10 +213,10 @@ pub fn sub() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(lt(), 0.into(), 0.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(lt(), 1.into(), 1.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(lt(), 0.into(), 1.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(lt(), 1.into(), 0.into()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(lt(), 0.into(), 0.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(lt(), 1.into(), 1.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(lt(), 0.into(), 1.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(lt(), 1.into(), 0.into()), NOR, 0), fls());
 /// # }
 /// ```
 pub fn lt() -> Term {
@@ -239,10 +239,10 @@ pub fn lt() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(leq(), 0.into(), 0.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(leq(), 1.into(), 1.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(leq(), 0.into(), 1.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(leq(), 1.into(), 0.into()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(leq(), 0.into(), 0.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(leq(), 1.into(), 1.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(leq(), 0.into(), 1.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(leq(), 1.into(), 0.into()), NOR, 0), fls());
 /// # }
 /// ```
 pub fn leq() -> Term {
@@ -265,10 +265,10 @@ pub fn leq() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(eq(), 0.into(), 0.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(eq(), 1.into(), 1.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(eq(), 0.into(), 1.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(eq(), 1.into(), 0.into()), &Normal, 0), fls());
+/// assert_eq!(beta(app!(eq(), 0.into(), 0.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(eq(), 1.into(), 1.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(eq(), 0.into(), 1.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(eq(), 1.into(), 0.into()), NOR, 0), fls());
 /// # }
 /// ```
 pub fn eq() -> Term {
@@ -295,10 +295,10 @@ pub fn eq() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(neq(), 0.into(), 0.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(neq(), 1.into(), 1.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(neq(), 0.into(), 1.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(neq(), 1.into(), 0.into()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(neq(), 0.into(), 0.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(neq(), 1.into(), 1.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(neq(), 0.into(), 1.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(neq(), 1.into(), 0.into()), NOR, 0), tru());
 /// # }
 /// ```
 pub fn neq() -> Term {
@@ -325,10 +325,10 @@ pub fn neq() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(geq(), 0.into(), 0.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(geq(), 1.into(), 1.into()), &Normal, 0), tru());
-/// assert_eq!(beta(app!(geq(), 0.into(), 1.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(geq(), 1.into(), 0.into()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(geq(), 0.into(), 0.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(geq(), 1.into(), 1.into()), NOR, 0), tru());
+/// assert_eq!(beta(app!(geq(), 0.into(), 1.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(geq(), 1.into(), 0.into()), NOR, 0), tru());
 /// # }
 /// ```
 pub fn geq() -> Term {
@@ -351,10 +351,10 @@ pub fn geq() -> Term {
 /// use lambda_calculus::reduction::beta;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// assert_eq!(beta(app!(gt(), 0.into(), 0.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(gt(), 1.into(), 1.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(gt(), 0.into(), 1.into()), &Normal, 0), fls());
-/// assert_eq!(beta(app!(gt(), 1.into(), 0.into()), &Normal, 0), tru());
+/// assert_eq!(beta(app!(gt(), 0.into(), 0.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(gt(), 1.into(), 1.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(gt(), 0.into(), 1.into()), NOR, 0), fls());
+/// assert_eq!(beta(app!(gt(), 1.into(), 0.into()), NOR, 0), tru());
 /// # }
 /// ```
 pub fn gt() -> Term {
@@ -366,7 +366,7 @@ pub fn gt() -> Term {
 /// Applied to two Church-encoded numbers it returns a Church-encoded pair with the result of their
 /// division - the quotient and the remainder. It loops indefinitely if the divisor is `zero()`.
 ///
-/// DIV := Z (λgqab.LT a b (λx.PAIR q a) (λx.g (SUCC q) (SUB a b) b) I) ZERO =
+/// DIV := Z (λzqab.LT a b (λx.PAIR q a) (λx.z (SUCC q) (SUB a b) b) I) ZERO =
 /// Z (λ λ λ λ LT 2 1 (λ PAIR 4 3) (λ 5 (SUCC 4) (SUB 3 2) 2) I) ZERO
 ///
 /// # Example
@@ -378,7 +378,7 @@ pub fn gt() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(div(), 5.into(), 2.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, Term::from((2.into(), 1.into())));
 /// # }
@@ -408,7 +408,7 @@ pub fn div() -> Term {
 /// Applied to two Church-encoded numbers it returns a Church-encoded quotient of their division.
 /// It loops indefinitely if the second argument is `zero()`.
 ///
-/// QUOT := Z (λrab.LT a b (λx.ZERO) (λx.SUCC (r (SUB a b) b)) I) =
+/// QUOT := Z (λzab.LT a b (λx.ZERO) (λx.SUCC (z (SUB a b) b)) I) =
 /// Z (λ λ λ LT 2 1 (λ ZERO) (λ SUCC (4 (SUB 3 2) 2)) I)
 ///
 /// # Example
@@ -419,7 +419,7 @@ pub fn div() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(quot(), 6.into(), 2.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 3.into());
 /// # }
@@ -447,11 +447,10 @@ pub fn quot() -> Term {
     )
 }
 
-// TODO: find an independent variant, like with quot()
 /// Applied to two Church-encoded numbers it returns a Church-encoded remainder of their division.
 /// It loops indefinitely if the second argument is `zero()`.
 ///
-/// REM := λab. SECOND (DIV a b) = λ λ SECOND (DIV 2 1)
+/// REM := Z (λzab.LT a b (λx.a) (λx.z (SUB a b) b) I) = Z (λ λ λ LT 2 1 (λ 3) (λ 4 (SUB 3 2) 2) I)
 ///
 /// # Example
 /// ```
@@ -461,33 +460,49 @@ pub fn quot() -> Term {
 /// use lambda_calculus::reduction::Order::*;
 ///
 /// let mut expr = app!(rem(), 3.into(), 2.into());
-/// expr.beta(&Normal, 0);
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 1.into());
 /// # }
 /// ```
 pub fn rem() -> Term {
-    abs(abs(
-        app(second(), app!(div(), Var(2), Var(1)))
-    ))
+    app(
+        z(),
+        abs(abs(abs(
+            app!(
+                lt(),
+                Var(2),
+                Var(1),
+                abs(Var(3)),
+                abs(
+                    app!(
+                        Var(4),
+                        app!(sub(), Var(3), Var(2)),
+                        Var(2)
+                    )
+                ),
+                i()
+            )
+        )))
+    )
 }
 
 /// Applied to a Church-encoded number it yields its Church-encoded factorial.
 ///
-/// FACTORIAL := λn. n (λfax. f (MULT a x) (SUCC x)) K ONE ONE =
+/// FAC := λn. n (λfab. f (MULT a b) (SUCC b)) K ONE ONE =
 /// λ 1 (λ λ λ 3 (MULT 2 1) (SUCC 1)) K ONE ONE
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::arithmetic::factorial;
+/// use lambda_calculus::arithmetic::fac;
 /// use lambda_calculus::reduction::Order::*;
 ///
-/// let mut expr = factorial().app(3.into());
-/// expr.beta(&Normal, 0);
+/// let mut expr = fac().app(3.into());
+/// expr.beta(NOR, 0);
 ///
 /// assert_eq!(expr, 6.into());
 /// ```
-pub fn factorial() -> Term {
+pub fn fac() -> Term {
     abs(
         app!(
             Var(1),
@@ -582,184 +597,193 @@ mod test {
 
     #[test]
     fn church_successor() {
-        assert_eq!(beta(app!(succ(), 0.into()), &Normal, 0), 1.into());
-        assert_eq!(beta(app!(succ(), 1.into()), &Normal, 0), 2.into());
-        assert_eq!(beta(app!(succ(), 2.into()), &Normal, 0), 3.into());
+        assert_eq!(beta(app!(succ(), 0.into()), NOR, 0), 1.into());
+        assert_eq!(beta(app!(succ(), 1.into()), NOR, 0), 2.into());
+        assert_eq!(beta(app!(succ(), 2.into()), NOR, 0), 3.into());
 
-        assert_eq!(beta(app!(succ(), 0.into()), &HybridNormal, 0), 1.into());
-        assert_eq!(beta(app!(succ(), 1.into()), &HybridNormal, 0), 2.into());
-        assert_eq!(beta(app!(succ(), 2.into()), &HybridNormal, 0), 3.into());
+        assert_eq!(beta(app!(succ(), 0.into()), HNO, 0), 1.into());
+        assert_eq!(beta(app!(succ(), 1.into()), HNO, 0), 2.into());
+        assert_eq!(beta(app!(succ(), 2.into()), HNO, 0), 3.into());
 
-        assert_eq!(beta(app!(succ(), 0.into()), &Applicative, 0), 1.into());
-        assert_eq!(beta(app!(succ(), 1.into()), &Applicative, 0), 2.into());
-        assert_eq!(beta(app!(succ(), 2.into()), &Applicative, 0), 3.into());
+        assert_eq!(beta(app!(succ(), 0.into()), APP, 0), 1.into());
+        assert_eq!(beta(app!(succ(), 1.into()), APP, 0), 2.into());
+        assert_eq!(beta(app!(succ(), 2.into()), APP, 0), 3.into());
 
-        assert_eq!(beta(app!(succ(), 0.into()), &HybridApplicative, 0), 1.into());
-        assert_eq!(beta(app!(succ(), 1.into()), &HybridApplicative, 0), 2.into());
-        assert_eq!(beta(app!(succ(), 2.into()), &HybridApplicative, 0), 3.into());
+        assert_eq!(beta(app!(succ(), 0.into()), HAP, 0), 1.into());
+        assert_eq!(beta(app!(succ(), 1.into()), HAP, 0), 2.into());
+        assert_eq!(beta(app!(succ(), 2.into()), HAP, 0), 3.into());
     }
 
     #[test]
     fn church_predecessor() {
-        assert_eq!(beta(app!(pred(), 0.into()), &Normal, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 1.into()), &Normal, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 5.into()), &Normal, 0), 4.into());
+        assert_eq!(beta(app!(pred(), 0.into()), NOR, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 1.into()), NOR, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 5.into()), NOR, 0), 4.into());
 
-        assert_eq!(beta(app!(pred(), 0.into()), &HybridNormal, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 1.into()), &HybridNormal, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 5.into()), &HybridNormal, 0), 4.into());
+        assert_eq!(beta(app!(pred(), 0.into()), HNO, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 1.into()), HNO, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 5.into()), HNO, 0), 4.into());
 
-        assert_eq!(beta(app!(pred(), 0.into()), &Applicative, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 1.into()), &Applicative, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 5.into()), &Applicative, 0), 4.into());
+        assert_eq!(beta(app!(pred(), 0.into()), APP, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 1.into()), APP, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 5.into()), APP, 0), 4.into());
 
-        assert_eq!(beta(app!(pred(), 0.into()), &HybridApplicative, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 1.into()), &HybridApplicative, 0), 0.into());
-        assert_eq!(beta(app!(pred(), 5.into()), &HybridApplicative, 0), 4.into());
+        assert_eq!(beta(app!(pred(), 0.into()), HAP, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 1.into()), HAP, 0), 0.into());
+        assert_eq!(beta(app!(pred(), 5.into()), HAP, 0), 4.into());
     }
 
     #[test]
     fn church_plus_sub_equivalents() {
         /* PLUS 1 → SUCC & C SUB 1 → PRED */
-        assert_eq!(beta(app!(    plus(), 1.into()), &Normal, 0), succ());
-        assert_eq!(beta(app!(c(), sub(), 1.into()), &Normal, 0), pred());
+        assert_eq!(beta(app!(    plus(), 1.into()), NOR, 0), succ());
+        assert_eq!(beta(app!(c(), sub(), 1.into()), NOR, 0), pred());
 
-        assert_eq!(beta(app!(    plus(), 1.into()), &HybridNormal, 0), succ());
-        assert_eq!(beta(app!(c(), sub(), 1.into()), &HybridNormal, 0), pred());
+        assert_eq!(beta(app!(    plus(), 1.into()), HNO, 0), succ());
+        assert_eq!(beta(app!(c(), sub(), 1.into()), HNO, 0), pred());
 
-        assert_eq!(beta(app!(    plus(), 1.into()), &Applicative, 0), succ());
-        assert_eq!(beta(app!(c(), sub(), 1.into()), &Applicative, 0), pred());
+        assert_eq!(beta(app!(    plus(), 1.into()), APP, 0), succ());
+        assert_eq!(beta(app!(c(), sub(), 1.into()), APP, 0), pred());
 
-        assert_eq!(beta(app!(    plus(), 1.into()), &HybridApplicative, 0), succ());
-        assert_eq!(beta(app!(c(), sub(), 1.into()), &HybridApplicative, 0), pred());
+        assert_eq!(beta(app!(    plus(), 1.into()), HAP, 0), succ());
+        assert_eq!(beta(app!(c(), sub(), 1.into()), HAP, 0), pred());
     }
 
     #[test]
     fn church_multiplication() {
-        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), &Normal, 0), 12.into());
-        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), &Normal, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), &Normal, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), &Normal, 0),  0.into());
-        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), &Normal, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), NOR, 0), 12.into());
+        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), NOR, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), NOR, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), NOR, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), NOR, 0),  0.into());
 
-        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), &HybridNormal, 0), 12.into());
-        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), &HybridNormal, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), &HybridNormal, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), &HybridNormal, 0),  0.into());
-        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), &HybridNormal, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), HNO, 0), 12.into());
+        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), HNO, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), HNO, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), HNO, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), HNO, 0),  0.into());
 
-        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), &Applicative, 0), 12.into());
-        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), &Applicative, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), &Applicative, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), &Applicative, 0),  0.into());
-        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), &Applicative, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), APP, 0), 12.into());
+        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), APP, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), APP, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), APP, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), APP, 0),  0.into());
 
-        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), &HybridApplicative, 0), 12.into());
-        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), &HybridApplicative, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), &HybridApplicative, 0),  3.into());
-        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), &HybridApplicative, 0),  0.into());
-        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), &HybridApplicative, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 4.into()), HAP, 0), 12.into());
+        assert_eq!(beta(app!(mult(), 1.into(), 3.into()), HAP, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 3.into(), 1.into()), HAP, 0),  3.into());
+        assert_eq!(beta(app!(mult(), 5.into(), 0.into()), HAP, 0),  0.into());
+        assert_eq!(beta(app!(mult(), 0.into(), 5.into()), HAP, 0),  0.into());
     }
 
     #[test]
     fn church_exponentiation() {
-        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), &Normal, 0), 16.into());
-        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), &Normal, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), &Normal, 0),  3.into());
-        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), &Normal, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), &Normal, 0),  0.into());
+        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), NOR, 0), 16.into());
+        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), NOR, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), NOR, 0),  3.into());
+        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), NOR, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), NOR, 0),  0.into());
 
-        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), &HybridNormal, 0), 16.into());
-        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), &HybridNormal, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), &HybridNormal, 0),  3.into());
-        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), &HybridNormal, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), &HybridNormal, 0),  0.into());
+        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), HNO, 0), 16.into());
+        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), HNO, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), HNO, 0),  3.into());
+        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), HNO, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), HNO, 0),  0.into());
 
-        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), &Applicative, 0), 16.into());
-        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), &Applicative, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), &Applicative, 0),  3.into());
-        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), &Applicative, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), &Applicative, 0),  0.into());
+        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), APP, 0), 16.into());
+        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), APP, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), APP, 0),  3.into());
+        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), APP, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), APP, 0),  0.into());
 
-        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), &HybridApplicative, 0), 16.into());
-        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), &HybridApplicative, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), &HybridApplicative, 0),  3.into());
-        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), &HybridApplicative, 0),  1.into());
-        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), &HybridApplicative, 0),  0.into());
+        assert_eq!(beta(app!(pow(), 2.into(), 4.into()), HAP, 0), 16.into());
+        assert_eq!(beta(app!(pow(), 1.into(), 3.into()), HAP, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 3.into(), 1.into()), HAP, 0),  3.into());
+        assert_eq!(beta(app!(pow(), 5.into(), 0.into()), HAP, 0),  1.into());
+        assert_eq!(beta(app!(pow(), 0.into(), 5.into()), HAP, 0),  0.into());
     }
 
     #[test]
     fn church_division() {
-        assert_eq!(beta(app!(div(), 2.into(), 2.into()), &Normal, 0), (1.into(), 0.into()).into());
-        assert_eq!(beta(app!(div(), 3.into(), 2.into()), &Normal, 0), (1.into(), 1.into()).into());
-        assert_eq!(beta(app!(div(), 2.into(), 1.into()), &Normal, 0), (2.into(), 0.into()).into());
-        assert_eq!(beta(app!(div(), 0.into(), 3.into()), &Normal, 0), (0.into(), 0.into()).into());
-     // assert_eq!(beta(app!(div(), 1.into(), 0.into()), &Normal, 0), ); division by 0 hangs
+        assert_eq!(beta(app!(div(), 2.into(), 2.into()), NOR, 0), (1.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 3.into(), 2.into()), NOR, 0), (1.into(), 1.into()).into());
+        assert_eq!(beta(app!(div(), 5.into(), 2.into()), NOR, 0), (2.into(), 1.into()).into());
+        assert_eq!(beta(app!(div(), 2.into(), 1.into()), NOR, 0), (2.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 0.into(), 3.into()), NOR, 0), (0.into(), 0.into()).into());
+     // assert_eq!(beta(app!(div(), 1.into(), 0.into()), NOR, 0), ); division by 0 hangs
 
-        assert_eq!(beta(app!(div(), 2.into(), 2.into()), &HybridNormal, 0), (1.into(), 0.into()).into());
-        assert_eq!(beta(app!(div(), 3.into(), 2.into()), &HybridNormal, 0), (1.into(), 1.into()).into());
-        assert_eq!(beta(app!(div(), 2.into(), 1.into()), &HybridNormal, 0), (2.into(), 0.into()).into());
-        assert_eq!(beta(app!(div(), 0.into(), 3.into()), &HybridNormal, 0), (0.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 2.into(), 2.into()), HNO, 0), (1.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 3.into(), 2.into()), HNO, 0), (1.into(), 1.into()).into());
+        assert_eq!(beta(app!(div(), 5.into(), 2.into()), HNO, 0), (2.into(), 1.into()).into());
+        assert_eq!(beta(app!(div(), 2.into(), 1.into()), HNO, 0), (2.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 0.into(), 3.into()), HNO, 0), (0.into(), 0.into()).into());
 
-        assert_eq!(beta(app!(div(), 2.into(), 2.into()), &HybridApplicative, 0), (1.into(), 0.into()).into());
-        assert_eq!(beta(app!(div(), 3.into(), 2.into()), &HybridApplicative, 0), (1.into(), 1.into()).into());
-        assert_eq!(beta(app!(div(), 2.into(), 1.into()), &HybridApplicative, 0), (2.into(), 0.into()).into());
-        assert_eq!(beta(app!(div(), 0.into(), 3.into()), &HybridApplicative, 0), (0.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 2.into(), 2.into()), HAP, 0), (1.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 3.into(), 2.into()), HAP, 0), (1.into(), 1.into()).into());
+        assert_eq!(beta(app!(div(), 5.into(), 2.into()), HAP, 0), (2.into(), 1.into()).into());
+        assert_eq!(beta(app!(div(), 2.into(), 1.into()), HAP, 0), (2.into(), 0.into()).into());
+        assert_eq!(beta(app!(div(), 0.into(), 3.into()), HAP, 0), (0.into(), 0.into()).into());
     }
 
     #[test]
     fn church_quotient() {
-        assert_eq!(beta(app!(quot(), 2.into(), 2.into()), &Normal, 0), 1.into());
-        assert_eq!(beta(app!(quot(), 3.into(), 2.into()), &Normal, 0), 1.into());
-        assert_eq!(beta(app!(quot(), 2.into(), 1.into()), &Normal, 0), 2.into());
-        assert_eq!(beta(app!(quot(), 0.into(), 3.into()), &Normal, 0), 0.into());
-    //  assert_eq!(beta(app!(quot(), 1.into(), 0.into()), &Normal, 0), ); division by 0 hangs
+        assert_eq!(beta(app!(quot(), 2.into(), 2.into()), NOR, 0), 1.into());
+        assert_eq!(beta(app!(quot(), 3.into(), 2.into()), NOR, 0), 1.into());
+        assert_eq!(beta(app!(quot(), 5.into(), 2.into()), NOR, 0), 2.into());
+        assert_eq!(beta(app!(quot(), 2.into(), 1.into()), NOR, 0), 2.into());
+        assert_eq!(beta(app!(quot(), 0.into(), 3.into()), NOR, 0), 0.into());
+    //  assert_eq!(beta(app!(quot(), 1.into(), 0.into()), NOR, 0), ); division by 0 hangs
 
-        assert_eq!(beta(app!(quot(), 2.into(), 2.into()), &HybridNormal, 0), 1.into());
-        assert_eq!(beta(app!(quot(), 3.into(), 2.into()), &HybridNormal, 0), 1.into());
-        assert_eq!(beta(app!(quot(), 2.into(), 1.into()), &HybridNormal, 0), 2.into());
-        assert_eq!(beta(app!(quot(), 0.into(), 3.into()), &HybridNormal, 0), 0.into());
+        assert_eq!(beta(app!(quot(), 2.into(), 2.into()), HNO, 0), 1.into());
+        assert_eq!(beta(app!(quot(), 3.into(), 2.into()), HNO, 0), 1.into());
+        assert_eq!(beta(app!(quot(), 5.into(), 2.into()), HNO, 0), 2.into());
+        assert_eq!(beta(app!(quot(), 2.into(), 1.into()), HNO, 0), 2.into());
+        assert_eq!(beta(app!(quot(), 0.into(), 3.into()), HNO, 0), 0.into());
 
-        assert_eq!(beta(app!(quot(), 2.into(), 2.into()), &HybridApplicative, 0), 1.into());
-        assert_eq!(beta(app!(quot(), 3.into(), 2.into()), &HybridApplicative, 0), 1.into());
-        assert_eq!(beta(app!(quot(), 2.into(), 1.into()), &HybridApplicative, 0), 2.into());
-        assert_eq!(beta(app!(quot(), 0.into(), 3.into()), &HybridApplicative, 0), 0.into());
+        assert_eq!(beta(app!(quot(), 2.into(), 2.into()), HAP, 0), 1.into());
+        assert_eq!(beta(app!(quot(), 3.into(), 2.into()), HAP, 0), 1.into());
+        assert_eq!(beta(app!(quot(), 5.into(), 2.into()), HAP, 0), 2.into());
+        assert_eq!(beta(app!(quot(), 2.into(), 1.into()), HAP, 0), 2.into());
+        assert_eq!(beta(app!(quot(), 0.into(), 3.into()), HAP, 0), 0.into());
     }
 
     #[test]
     fn church_remainder() {
-        assert_eq!(beta(app!(rem(), 2.into(), 2.into()), &Normal, 0), 0.into());
-        assert_eq!(beta(app!(rem(), 3.into(), 2.into()), &Normal, 0), 1.into());
-        assert_eq!(beta(app!(rem(), 2.into(), 1.into()), &Normal, 0), 0.into());
-        assert_eq!(beta(app!(rem(), 0.into(), 3.into()), &Normal, 0), 0.into());
-     // assert_eq!(beta(app!(rem(), 1.into(), 0.into()), &Normal, 0), ); division by 0 hangs
+        assert_eq!(beta(app!(rem(), 2.into(), 2.into()), NOR, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 3.into(), 2.into()), NOR, 0), 1.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 5.into()), NOR, 0), 2.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 1.into()), NOR, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 0.into(), 3.into()), NOR, 0), 0.into());
+     // assert_eq!(beta(app!(rem(), 1.into(), 0.into()), NOR, 0), ); division by 0 hangs
 
-        assert_eq!(beta(app!(rem(), 2.into(), 2.into()), &HybridNormal, 0), 0.into());
-        assert_eq!(beta(app!(rem(), 3.into(), 2.into()), &HybridNormal, 0), 1.into());
-        assert_eq!(beta(app!(rem(), 2.into(), 1.into()), &HybridNormal, 0), 0.into());
-        assert_eq!(beta(app!(rem(), 0.into(), 3.into()), &HybridNormal, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 2.into()), HNO, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 3.into(), 2.into()), HNO, 0), 1.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 5.into()), HNO, 0), 2.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 1.into()), HNO, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 0.into(), 3.into()), HNO, 0), 0.into());
 
-        assert_eq!(beta(app!(rem(), 2.into(), 2.into()), &HybridApplicative, 0), 0.into());
-        assert_eq!(beta(app!(rem(), 3.into(), 2.into()), &HybridApplicative, 0), 1.into());
-        assert_eq!(beta(app!(rem(), 2.into(), 1.into()), &HybridApplicative, 0), 0.into());
-        assert_eq!(beta(app!(rem(), 0.into(), 3.into()), &HybridApplicative, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 2.into()), HAP, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 3.into(), 2.into()), HAP, 0), 1.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 5.into()), HAP, 0), 2.into());
+        assert_eq!(beta(app!(rem(), 2.into(), 1.into()), HAP, 0), 0.into());
+        assert_eq!(beta(app!(rem(), 0.into(), 3.into()), HAP, 0), 0.into());
     }
 
     #[test]
-    fn church_factorial() {
-        assert_eq!(beta(app!(factorial(), 0.into()), &Normal, 0), 1.into());
-        assert_eq!(beta(app!(factorial(), 1.into()), &Normal, 0), 1.into());
-        assert_eq!(beta(app!(factorial(), 2.into()), &Normal, 0), 2.into());
-        assert_eq!(beta(app!(factorial(), 3.into()), &Normal, 0), 6.into());
+    fn church_fac() {
+        assert_eq!(beta(app!(fac(), 0.into()), NOR, 0), 1.into());
+        assert_eq!(beta(app!(fac(), 1.into()), NOR, 0), 1.into());
+        assert_eq!(beta(app!(fac(), 2.into()), NOR, 0), 2.into());
+        assert_eq!(beta(app!(fac(), 3.into()), NOR, 0), 6.into());
 
-        assert_eq!(beta(app!(factorial(), 0.into()), &HybridNormal, 0), 1.into());
-        assert_eq!(beta(app!(factorial(), 1.into()), &HybridNormal, 0), 1.into());
-        assert_eq!(beta(app!(factorial(), 2.into()), &HybridNormal, 0), 2.into());
-        assert_eq!(beta(app!(factorial(), 3.into()), &HybridNormal, 0), 6.into());
+        assert_eq!(beta(app!(fac(), 0.into()), HNO, 0), 1.into());
+        assert_eq!(beta(app!(fac(), 1.into()), HNO, 0), 1.into());
+        assert_eq!(beta(app!(fac(), 2.into()), HNO, 0), 2.into());
+        assert_eq!(beta(app!(fac(), 3.into()), HNO, 0), 6.into());
 
-        assert_eq!(beta(app!(factorial(), 0.into()), &HybridApplicative, 0), 1.into());
-        assert_eq!(beta(app!(factorial(), 1.into()), &HybridApplicative, 0), 1.into());
-        assert_eq!(beta(app!(factorial(), 2.into()), &HybridApplicative, 0), 2.into());
-        assert_eq!(beta(app!(factorial(), 3.into()), &HybridApplicative, 0), 6.into());
+        assert_eq!(beta(app!(fac(), 0.into()), HAP, 0), 1.into());
+        assert_eq!(beta(app!(fac(), 1.into()), HAP, 0), 1.into());
+        assert_eq!(beta(app!(fac(), 2.into()), HAP, 0), 2.into());
+        assert_eq!(beta(app!(fac(), 3.into()), HAP, 0), 6.into());
     }
 }
