@@ -84,7 +84,7 @@ impl Term {
     /// # }
     /// ```
     pub fn is_pair(&self) -> bool {
-        self.fst_ref().is_ok() && self.snd_ref().is_ok()
+        self.unpair_ref().is_ok()
     }
 
     /// Splits a Church-encoded pair into a pair of terms, consuming `self`.
@@ -102,18 +102,12 @@ impl Term {
     /// # }
     /// ```
     pub fn unpair(self) -> Result<(Term, Term), Error> {
-        if let Abs(abstracted) = self {
-            if let Ok((wrapped_a, b)) = abstracted.unapp() {
-                Ok((try!(wrapped_a.rhs()), b))
-            } else {
-                Err(NotAPair)
-            }
+        let candidate = if let Abs(abstracted) = self { *abstracted } else { self };
+
+        if let Ok((wrapped_a, b)) = candidate.unapp() {
+            Ok((try!(wrapped_a.rhs()), b))
         } else {
-            if let Ok((wrapped_a, b)) = self.unapp() {
-                Ok((try!(wrapped_a.rhs()), b))
-            } else {
-                Err(NotAPair)
-            }
+            Err(NotAPair)
         }
     }
 
@@ -132,18 +126,12 @@ impl Term {
     /// # }
     /// ```
     pub fn unpair_ref(&self) -> Result<(&Term, &Term), Error> {
-        if let Abs(ref abstracted) = *self {
-            if let Ok((wrapped_a, b)) = abstracted.unapp_ref() {
-                Ok((try!(wrapped_a.rhs_ref()), b))
-            } else {
-                Err(NotAPair)
-            }
+        let candidate = if let Abs(ref abstracted) = *self { abstracted } else { self };
+
+        if let Ok((wrapped_a, b)) = candidate.unapp_ref() {
+            Ok((try!(wrapped_a.rhs_ref()), b))
         } else {
-            if let Ok((wrapped_a, b)) = self.unapp_ref() {
-                Ok((try!(wrapped_a.rhs_ref()), b))
-            } else {
-                Err(NotAPair)
-            }
+            Err(NotAPair)
         }
     }
 
@@ -162,18 +150,12 @@ impl Term {
     /// # }
     /// ```
     pub fn unpair_ref_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
-        if let Abs(ref mut abstracted) = *self {
-            if let Ok((wrapped_a, b)) = abstracted.unapp_ref_mut() {
-                Ok((try!(wrapped_a.rhs_ref_mut()), b))
-            } else {
-                Err(NotAPair)
-            }
+        let mut candidate = if let Abs(ref mut abstracted) = *self { abstracted } else { self };
+
+        if let Ok((wrapped_a, b)) = candidate.unapp_ref_mut() {
+            Ok((try!(wrapped_a.rhs_ref_mut()), b))
         } else {
-            if let Ok((wrapped_a, b)) = self.unapp_ref_mut() {
-                Ok((try!(wrapped_a.rhs_ref_mut()), b))
-            } else {
-                Err(NotAPair)
-            }
+            Err(NotAPair)
         }
     }
 
