@@ -1,9 +1,10 @@
-//! A parser for lambda expressions with
-//! [De Bruijn indices](https://en.wikipedia.org/wiki/De_Bruijn_index)
+//! A parser for lambda expressions
 
 use term::*;
 use term::Term::*;
+use term::Notation::*;
 use self::Token::*;
+use self::CToken::*;
 use self::Error::*;
 use self::Expression::*;
 
@@ -23,12 +24,19 @@ enum Token {
     Number(usize)
 }
 
-fn tokenize(input: &str) -> Result<Vec<Token>, Error> {
-    let mut chars = input.chars();
-    let mut tokens = Vec::new();
-    let mut position = 0;
+#[derive(Debug, PartialEq)]
+enum CToken {
+    CLambda(String),
+    CLparen,
+    CRparen,
+    CName(String)
+}
 
-    while let Some(c) = chars.next() {
+fn tokenize_dbr(input: &str) -> Result<Vec<Token>, Error> {
+    let mut chars = input.chars().enumerate();
+    let mut tokens = Vec::new();
+
+    while let Some((i, c)) = chars.next() {
         match c {
      '\\' | 'λ' => { tokens.push(Lambda) },
             '(' => { tokens.push(Lparen) },
@@ -39,16 +47,101 @@ fn tokenize(input: &str) -> Result<Vec<Token>, Error> {
                 } else if x.is_whitespace() {
                     ()
                 } else {
-                    return Err(InvalidCharacter((position, x)))
+                    return Err(InvalidCharacter((i + 1, x)))
                 }
             }
         }
-        position += if c == 'λ' { 2 } else { 1 };
     }
 
     Ok(tokens)
 }
 
+fn tokenize_cla(input: &str) -> Result<Vec<CToken>, Error> {
+    let mut chars = input.chars().enumerate().peekable();
+    let mut tokens = Vec::new();
+
+    while let Some((i, c)) = chars.next() {
+        match c {
+            '\\' | 'λ' => {
+                let mut name = String::new();
+                while let Some((i, c)) = chars.next() {
+                    if c == '.' {
+                        break
+                    } else if "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".contains(c) {
+                        name.push(c)
+                    } else {
+                        return Err(InvalidCharacter((i + 1, c)))
+                    }
+                }
+                tokens.push(CLambda(name))
+            },
+            '(' => { tokens.push(CLparen) },
+            ')' => { tokens.push(CRparen) },
+             x  => {
+                if x.is_whitespace() {
+                    ()
+                } else if "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".contains(x) {
+                    let mut name = x.to_string();
+                    while let Some(&(_, c)) = chars.peek() {
+                        if c.is_whitespace() || c == '(' || c == ')' {
+                            break
+                        } else {
+                            name.push(c);
+                            chars.next();
+                        }
+                    }
+                    tokens.push(CName(name))
+                } else {
+                    return Err(InvalidCharacter((i + 1, x)))
+                }
+            }
+        }
+    }
+
+    Ok(tokens)
+}
+
+fn convert_classic_tokens(tokens: &[CToken]) -> Vec<Token> {
+    _convert_classic_tokens(tokens, &mut Vec::new(), &mut 0)
+}
+
+fn _convert_classic_tokens(tokens: &[CToken], stack: &mut Vec<String>, pos: &mut usize) -> Vec<Token>
+{
+    let mut output = Vec::new();
+    let mut inner_stack_count = 0;
+
+    while *pos < tokens.len() {
+        match tokens[*pos] {
+            CLambda(ref name) => {
+                output.push(Lambda);
+                stack.push(name.clone());
+                inner_stack_count += 1;
+            },
+            CLparen => {
+                output.push(Lparen);
+                *pos += 1;
+                output.append(&mut _convert_classic_tokens(tokens, stack, pos));
+            },
+            CRparen => {
+                output.push(Rparen);
+                for _ in 0..inner_stack_count { stack.pop(); }
+                return output
+            },
+            CName(ref name) => {
+                let number = if stack.contains(name) {
+                    stack.iter().rev().position(|t| t == name).unwrap() + 1
+                } else {
+                    stack.len() + 1
+                };
+                output.push(Number(number));
+            }
+        }
+        *pos += 1;
+    }
+
+    output
+}
+
 #[derive(Debug, PartialEq)]
 enum Expression {
     Abstraction,
@@ -56,6 +149,10 @@ enum Expression {
     Variable(usize)
 }
 
+fn get_ast(tokens: &[Token]) -> Result<Expression, Error> {
+    _get_ast(tokens, &mut 0)
+}
+
 fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
     let mut expr = Vec::new();
 
@@ -84,27 +181,33 @@ fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
     Ok(Sequence(expr))
 }
 
-fn get_ast(tokens: &[Token]) -> Result<Expression, Error> {
-    let mut pos = 0;
-
-    _get_ast(tokens, &mut pos)
-}
-
-/// Parses the input `&str` as a lambda `Term`. The lambdas can be represented either with the
-/// greek letter (λ) or a backslash (\\ - less aesthetic, but only one byte in size), the De Bruijn
-/// indices start with 1 and are hexadecimal digits, and whitespaces are ignored.
+/// Parses the input `&str` as a lambda `Term`.
+///
+/// - lambdas can be represented either with the greek letter (λ) or a backslash (\\ -
+/// less aesthetic, but only one byte in size)
+/// - the identifiers in `Classic` mode are `String`s of ASCII alphabetic characters
+/// - `Classic` mode ignores whitespaces where unambiguous
+/// - `DeBruijn` mode ignores all whitespaces (since indices > 15 are very unlikely)
+/// - the indices in the `DeBruijn` notation mode start with 1 and are hexadecimal digits
 ///
 /// # Example
 /// ```
 /// use lambda_calculus::parser::parse;
+/// use lambda_calculus::term::Notation::*;
 /// use lambda_calculus::arithmetic::{succ, pred};
 ///
-/// assert_eq!(parse(&"λ λ λ 2 (3 2 1)"), Ok(succ()));
-/// assert_eq!(parse(&r#"\ \ \ 2 (3 2 1)"#), Ok(succ()));
-/// assert_eq!(parse(&"λλλ3(λλ1(24))(λ2)(λ1)"), Ok(pred()));
+/// assert_eq!(parse(&"λn. λf. λx. n (λg. λh. h (g f)) (λu. x) (λu. u)", Classic), Ok(pred()));
+/// assert_eq!(parse(&"λn.λf.λx.n(λg.λh.h(g f))(λu.x)(λu.u)", Classic), Ok(pred()));
+///
+/// assert_eq!(parse(  &"λλλ2(321)",  DeBruijn), Ok(succ()));
+/// assert_eq!(parse(&r#"\\\2(321)"#, DeBruijn), Ok(succ()));
 /// ```
-pub fn parse(input: &str) -> Result<Term, Error> {
-    let tokens = try!(tokenize(input));
+pub fn parse(input: &str, notation: Notation) -> Result<Term, Error> {
+    let tokens = if notation == DeBruijn {
+        try!(tokenize_dbr(input))
+    } else {
+        convert_classic_tokens(&try!(tokenize_cla(input)))
+    };
     let ast = try!(get_ast(&tokens));
 
     let exprs = try!(if let Sequence(exprs) = ast { Ok(exprs) } else { Err(InvalidExpression) });
@@ -160,13 +263,14 @@ mod test {
 
     #[test]
     fn tokenization_error() {
-        assert_eq!(tokenize(&"λλx2"), Err(InvalidCharacter((4, 'x'))))
+        assert_eq!(tokenize_dbr(&"λλx2"),    Err(InvalidCharacter((3, 'x'))));
+        assert_eq!(tokenize_cla(&"λa.λb a"), Err(InvalidCharacter((6, ' '))));
     }
 
     #[test]
     fn tokenization_success() {
         let quine = "λ 1 ( (λ 1 1) (λ λ λ λ λ 1 4 (3 (5 5) 2) ) ) 1";
-        let tokens = tokenize(&quine);
+        let tokens = tokenize_dbr(&quine);
 
         assert!(tokens.is_ok());
         assert_eq!(tokens.unwrap(), vec![Lambda, Number(1), Lparen, Lparen, Lambda, Number(1),
@@ -175,14 +279,29 @@ mod test {
             Rparen, Rparen, Rparen, Number(1)]);
     }
 
+    #[test]
+    fn tokenization_success_classic() {
+        let blc_dbr = "(λ11)(λλλ1(λλλλ3(λ5(3(λ2(3(λλ3(λ123)))(4(λ4(λ31(21))))))(1(2(λ12))\
+            (λ4(λ4(λ2(14)))5))))(33)2)(λ1((λ11)(λ11)))";
+        let blc_cla = format!("{}", parse(&blc_dbr, DeBruijn).unwrap());
+
+        let tokens_cla = tokenize_cla(&blc_cla);
+        let tokens_dbr = tokenize_dbr(&blc_dbr);
+
+        assert!(tokens_cla.is_ok());
+        assert!(tokens_dbr.is_ok());
+
+        assert_eq!(convert_classic_tokens(&tokens_cla.unwrap()), tokens_dbr.unwrap());
+    }
+
     #[test]
     fn alternative_lambda_parsing() {
-        assert_eq!(parse(&"\\\\\\2(321)"), parse(&"λλλ2(321)"))
+        assert_eq!(parse(&"\\\\\\2(321)", DeBruijn), parse(&"λλλ2(321)", DeBruijn))
     }
 
     #[test]
     fn succ_ast() {
-        let tokens = tokenize(&"λλλ2(321)").unwrap();
+        let tokens = tokenize_dbr(&"λλλ2(321)").unwrap();
         let ast = get_ast(&tokens);
 
         assert_eq!(ast,
@@ -203,7 +322,7 @@ mod test {
     #[test]
     fn parse_y() {
         let y = "λ(λ2(11))(λ2(11))";
-        assert_eq!(parse(&y).expect("parsing Y failed!"),
+        assert_eq!(parse(&y, DeBruijn).expect("parsing Y failed!"),
             abs(
                 app(
                     abs(
@@ -226,7 +345,7 @@ mod test {
     #[test]
     fn parse_quine() {
         let quine = "λ1((λ11)(λλλλλ14(3(55)2)))1";
-        assert_eq!(parse(&quine).expect("parsing QUINE failed!"),
+        assert_eq!(parse(&quine, DeBruijn).expect("parsing QUINE failed!"),
             abs(app(app(Var(1), app(abs(app(Var(1), Var(1))), abs(abs(abs(abs(abs(app(app(Var(1),
             Var(4)), app(app(Var(3), app(Var(5), Var(5))), Var(2)))))))))), Var(1)))
         );
@@ -236,7 +355,7 @@ mod test {
     fn parse_blc() {
         let blc = "(λ11)(λλλ1(λλλλ3(λ5(3(λ2(3(λλ3(λ123)))(4(λ4(λ31(21))))))(1(2(λ12))\
                    (λ4(λ4(λ2(14)))5))))(33)2)(λ1((λ11)(λ11)))";
-        assert_eq!(parse(&blc).expect("parsing BLC failed!"),
+        assert_eq!(parse(&blc, DeBruijn).expect("parsing BLC failed!"),
             app(app(abs(app(Var(1), Var(1))), abs(abs(abs(app(app(app(Var(1),
             abs(abs(abs(abs(app(Var(3), abs(app(app(Var(5), app(Var(3), abs(app(app(Var(2),
             app(Var(3), abs(abs(app(Var(3), abs(app(app(Var(1), Var(2)), Var(3)))))))), app(Var(4),
