@@ -45,10 +45,11 @@ pub enum Order {
 /// ```
 /// use lambda_calculus::reduction::apply;
 /// use lambda_calculus::parser::parse;
+/// use lambda_calculus::term::Notation::DeBruijn;
 ///
-/// let lhs    = parse(&"λλ42(λ13)").unwrap();
-/// let rhs    = parse(&"λ51").unwrap();
-/// let result = parse(&"λ3(λ61)(λ1(λ71))").unwrap();
+/// let lhs    = parse(&"λλ42(λ13)", DeBruijn).unwrap();
+/// let rhs    = parse(&"λ51", DeBruijn).unwrap();
+/// let result = parse(&"λ3(λ61)(λ1(λ71))", DeBruijn).unwrap();
 ///
 /// assert_eq!(apply(lhs, &rhs), Ok(result));
 /// ```
@@ -84,9 +85,9 @@ fn update_free_variables(term: &mut Term, added_depth: usize, own_depth: usize)
         Abs(ref mut abstracted) => {
             update_free_variables(abstracted, added_depth, own_depth + 1)
         },
-        App(ref mut lhs_lhs, ref mut lhs_rhs) => {
-            update_free_variables(lhs_lhs, added_depth, own_depth);
-            update_free_variables(lhs_rhs, added_depth, own_depth)
+        App(ref mut lhs, ref mut rhs) => {
+            update_free_variables(lhs, added_depth, own_depth);
+            update_free_variables(rhs, added_depth, own_depth)
         }
     }
 }
@@ -149,10 +150,11 @@ impl Term {
     /// # Example
     /// ```
     /// use lambda_calculus::parser::parse;
+    /// use lambda_calculus::term::Notation::DeBruijn;
     ///
-    /// let lhs    = parse(&"λλ42(λ13)").unwrap();
-    /// let rhs    = parse(&"λ51").unwrap();
-    /// let result = parse(&"λ3(λ61)(λ1(λ71))").unwrap();
+    /// let lhs    = parse(&"λλ42(λ13)", DeBruijn).unwrap();
+    /// let rhs    = parse(&"λ51", DeBruijn).unwrap();
+    /// let result = parse(&"λ3(λ61)(λ1(λ71))", DeBruijn).unwrap();
     ///
     /// assert_eq!(lhs.apply(&rhs), Ok(result));
     /// ```
@@ -179,15 +181,14 @@ impl Term {
     }
 
     fn eval_with_info(&mut self, depth: u32, count: &usize) {
-        if SHOW_REDUCTIONS { print!("\n{}. {}\n", count + 1, show_precedence(self, 0, depth)) }
+        if SHOW_REDUCTIONS { print!("\n{}. {}\n", count + 1, show_precedence_cla(self, 0, depth)) }
 
         let copy = self.clone();
         *self = copy.eval().unwrap(); // safe; only called in reduction sites
 
         if SHOW_REDUCTIONS {
-            let mut indent_len = ((*count + 1) as f32).log10().trunc() as usize + 3;
-            if DISPLAY_CLASSIC { indent_len += 3 }
-            println!("=>{}{}", " ".repeat(indent_len), show_precedence(self, 0, depth))
+            let indent_len = ((*count + 1) as f32).log10().trunc() as usize + 5;
+            println!("=>{}{}", " ".repeat(indent_len), show_precedence_cla(self, 0, depth))
         }
     }
 
@@ -394,6 +395,7 @@ impl fmt::Display for Order {
 #[cfg(test)]
 mod test {
     use super::*;
+    use term::Notation::*;
     use parser::parse;
     use combinators::{i, omm};
     use arithmetic::fac;
@@ -401,12 +403,12 @@ mod test {
 
     #[test]
     fn normal_order() {
-        let reduces_instantly = parse(&"(λλ1)((λλλ((32)1))(λλ2))").unwrap();
+        let reduces_instantly = parse(&"(λλ1)((λλλ((32)1))(λλ2))", DeBruijn).unwrap();
         assert_eq!(beta(reduces_instantly.clone(), NOR, 0),
                    beta(reduces_instantly,         NOR, 1)
         );
 
-        let should_reduce = parse(&"(λ2)((λ111)(λ111))").unwrap();
+        let should_reduce = parse(&"(λ2)((λ111)(λ111))", DeBruijn).unwrap();
         assert_eq!(beta(should_reduce, NOR, 0), Var(1));
 
         let does_reduce = app(abs(Var(2)), omm());
